﻿#if !defined( CORE_H )
#define CORE_H
/* Shaderprogramozás - Grafika és játékfejlesztés DirectX-szel ----------------

	Állomány          : Core.h

    Nyelv             : C++

    Létrehozás dátuma : 2008.09.26

    Utolsó módosítás  : 2009.03.15

    Leírás            : Shaderprogramozás
						Grafika és játékfejlesztés DirectX-szel
						264. oldal
						
		 Alapvető lekérő és beállító függvények tárháza

               Copyright (C) Nyisztor Károly 2008-2009
							www.directx.hu
-----------------------------------------------------------------------------*/
/*- Befordított fejállományok -----------------------------------------------*/
#include "util.h"
using namespace app::debug;

#include "..\scene\Camera.h"
using namespace app::scene;

/*- Névterek ----------------------------------------------------------------*/
namespace app
{
namespace util
{
#pragma warning (disable : 4996)
/*- Definíciók és konstansok ------------------------------------------------*/
///<summary>
///mutató a Core osztály egyetlen példányára
///</summary>
///<remarks>
///define nélkül minden esetben, amikor használjuk a gyárat, ki kellene írni, hogy 
///Core::Instance().
///</remarks>
#if !defined( GameCore )
#define GameCore Core::Instance()
#endif

/*- Saját típusok és osztályok -----------------------------------------------*/
class Core
{	
// adattagok
private:
	/*! 
	 * \brief eszköz mutató
	 */
	LPDIRECT3DDEVICE9 myD3DDevicePtr;

	Camera* myCameraPtr;
	unsigned int myWindowWidth;
	unsigned int myWindowHeight;

	
	bool myFullScreenFlag;

	HWND myWinHandle;
	HINSTANCE myAppInstance;

// tagfüggvények
public:
	/*!
	 * \brief
	 * gyártófüggvény a singleton Core osztályhoz
	 * létrehozza az egyedüli Core példányt
	 */
	static Core* Core::Instance()
	{	
		// statikus példány létrehozása, a függvénybôl kilépve nem szûnik meg 
		static Core instance;
		return &instance;
	}
	
	Core::~Core()
	{
	}
	
	// lekérdezö
	inline LPDIRECT3DDEVICE9 Device() const
	{
		// nincs beállitva az eszközmutatónk
		if( !myD3DDevicePtr )
		{			
			FATAL_EXIT("Nincs beállitva az eszközmutató!")
		}
		return myD3DDevicePtr;
	}
	// beállitó
	void SetDevice( const LPDIRECT3DDEVICE9 devPtr_in )
	{
		// nincs beállitva az eszközmutatónk
		if( !devPtr_in )
		{			
			FATAL_EXIT("Az átadott eszközmutató érvénytelen!")
			return;
		}
		myD3DDevicePtr = devPtr_in;
	}
	// lekérdezö
	inline Camera* Camera() const
	{
		// nincs beállitva az eszközmutatónk
		if( !myCameraPtr )
		{			
			FATAL_EXIT("A Camera objektum nincs inicializálva!")
		}
		return myCameraPtr;
	}
	// beállitó
	void SetCamera( app::scene::Camera* cameraPtr_in )
	{
		// nincs beállitva az eszközmutatónk
		if( !cameraPtr_in )
		{
			FATAL_EXIT("Az átadott Camera mutató érvénytelen!")
		}
		myCameraPtr = cameraPtr_in;
	}	
	void SetWindowDim( const unsigned int width_in, const unsigned int height_in )
	{
		myWindowWidth = width_in;
		myWindowHeight = height_in;
	}
	unsigned int WinWidth() const
	{
		return myWindowWidth;
	}
	unsigned int WinHeight() const
	{
		return myWindowHeight;
	}
	void setFullScreen( bool flag_in )
	{
		myFullScreenFlag = flag_in;
	}
	const bool getFullScreen() const
	{
		return myFullScreenFlag;
	}

	void setHWND( const HWND hwnd_in )
	{
		myWinHandle = hwnd_in;
	}

	const HWND getHWND() const
	{
		return myWinHandle;
		if(!myWinHandle)
			FATAL_EXIT("Érvénytelen ablak kezelô (myWinHandle)!")
	}

	void setAppInstance(HINSTANCE hInst_in)
	{
		myAppInstance = hInst_in;
	}

	HINSTANCE getAppInstance()
	{
		if(!myAppInstance)
			FATAL_EXIT("Érvénytelen alkalmazás példány (myAppInstance)!")
		return myAppInstance;
	}

private:
	// elrejtjük a ctor-t, copy ctor-t és az értékadó operátort, 
	// megakadályozva ezzel az erôforrások közvetlen példányosítását
	Core() 
	{
		myD3DDevicePtr = NULL;
		myCameraPtr = NULL;
		myWindowWidth = 0;
		myWindowHeight = 0;
		myFullScreenFlag = false;
		myWinHandle = NULL;
		myAppInstance = NULL;
	};
	Core( const Core& );
	Core& operator =( const Core& );	
};

} // namespace util
} // namespace game

#endif